import sys

import js
import pyodide
from pyodide.ffi.wrappers import add_event_listener

from main import convert_nifti_to_glb  # type: ignore[attr-defined]


def on_click_help_button(_: None) -> None:
    help_dialog = js.document.getElementById("help-dialog")
    help_dialog.showModal()


async def on_click_convert_button(_: str) -> None:
    convert_button = js.document.getElementById("convert-button")
    convert_button.disabled = True
    laplacian_smoothing_iterations_input_range = js.document.getElementById(
        "laplacian-smoothing-iterations-input-range",
    )
    laplacian_smoothing_iterations_input_range.disabled = True
    load_nifti_file_input_file = js.document.getElementById(
        "load-nifti-file-input-file",
    )
    load_nifti_file_input_file.disabled = True
    marching_cubes_step_size_input_range = js.document.getElementById(
        "marching-cubes-step-size-input-range",
    )
    marching_cubes_step_size_input_range.disabled = True
    processing_div = js.document.getElementById("processing-div")
    processing_div.textContent = (
        "Converting NIfTI to GLB. It might take a few minutes..."
    )
    marching_cubes_step_size = int(marching_cubes_step_size_input_range.value)
    laplacian_smoothing_iterations = int(
        laplacian_smoothing_iterations_input_range.value,
    )
    file_list = load_nifti_file_input_file.files.to_py()
    if not file_list:
        return
    data = js.Uint8Array.new(await file_list.item(0).arrayBuffer())
    try:
        output = convert_nifti_to_glb(
            data,
            "output.glb",
            laplacian_smoothing_iterations,
            marching_cubes_step_size,
        )
    except Exception:  # noqa: BLE001
        convert_button.disabled = False
        load_nifti_file_input_file.disabled = False
        marching_cubes_step_size_input_range.disabled = False
        laplacian_smoothing_iterations_input_range.disabled = False
        processing_div.textContent = (
            "Out of memory. Try reducing the iterations and/or increasing the step size"
        )
        sys.exit()
    content = pyodide.ffi.to_js(output)
    a = js.document.createElement("a")
    js.document.body.appendChild(a)
    a.style = "display: none"
    blob = js.Blob.new([content])
    url = js.window.URL.createObjectURL(blob)
    a.href = url
    file_name_without_extension = ".".join(file_list.item(0).name.split(".")[:-1])
    a.download = f"{file_name_without_extension}-step-size-{marching_cubes_step_size}-iterations-{laplacian_smoothing_iterations}.glb"  # noqa: E501
    a.click()
    js.window.URL.revokeObjectURL(url)
    convert_button.disabled = False
    load_nifti_file_input_file.disabled = False
    marching_cubes_step_size_input_range.disabled = False
    laplacian_smoothing_iterations_input_range.disabled = False
    processing_div.textContent = "Conversion done."


def main() -> None:
    add_event_listener(
        js.document.getElementById("convert-button"),
        "click",
        on_click_convert_button,
    )
    add_event_listener(
        js.document.getElementById("help-button"),
        "click",
        on_click_help_button,
    )


if __name__ == "__main__":
    main()
